import { TestBed } from '@angular/core/testing';
import { HttpClientTestingModule, HttpTestingController } from '@angular/common/http/testing';

import { IVariableCategory } from '../variable-category.model';
import { sampleWithRequiredData, sampleWithNewData, sampleWithPartialData, sampleWithFullData } from '../variable-category.test-samples';

import { VariableCategoryService } from './variable-category.service';

const requireRestSample: IVariableCategory = {
  ...sampleWithRequiredData,
};

describe('VariableCategory Service', () => {
  let service: VariableCategoryService;
  let httpMock: HttpTestingController;
  let expectedResult: IVariableCategory | IVariableCategory[] | boolean | null;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule],
    });
    expectedResult = null;
    service = TestBed.inject(VariableCategoryService);
    httpMock = TestBed.inject(HttpTestingController);
  });

  describe('Service methods', () => {
    it('should find an element', () => {
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.find(123).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should create a VariableCategory', () => {
      const variableCategory = { ...sampleWithNewData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.create(variableCategory).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'POST' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should update a VariableCategory', () => {
      const variableCategory = { ...sampleWithRequiredData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.update(variableCategory).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PUT' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should partial update a VariableCategory', () => {
      const patchObject = { ...sampleWithPartialData };
      const returnedFromService = { ...requireRestSample };
      const expected = { ...sampleWithRequiredData };

      service.partialUpdate(patchObject).subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'PATCH' });
      req.flush(returnedFromService);
      expect(expectedResult).toMatchObject(expected);
    });

    it('should return a list of VariableCategory', () => {
      const returnedFromService = { ...requireRestSample };

      const expected = { ...sampleWithRequiredData };

      service.query().subscribe(resp => (expectedResult = resp.body));

      const req = httpMock.expectOne({ method: 'GET' });
      req.flush([returnedFromService]);
      httpMock.verify();
      expect(expectedResult).toMatchObject([expected]);
    });

    it('should delete a VariableCategory', () => {
      const expected = true;

      service.delete(123).subscribe(resp => (expectedResult = resp.ok));

      const req = httpMock.expectOne({ method: 'DELETE' });
      req.flush({ status: 200 });
      expect(expectedResult).toBe(expected);
    });

    describe('addVariableCategoryToCollectionIfMissing', () => {
      it('should add a VariableCategory to an empty array', () => {
        const variableCategory: IVariableCategory = sampleWithRequiredData;
        expectedResult = service.addVariableCategoryToCollectionIfMissing([], variableCategory);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(variableCategory);
      });

      it('should not add a VariableCategory to an array that contains it', () => {
        const variableCategory: IVariableCategory = sampleWithRequiredData;
        const variableCategoryCollection: IVariableCategory[] = [
          {
            ...variableCategory,
          },
          sampleWithPartialData,
        ];
        expectedResult = service.addVariableCategoryToCollectionIfMissing(variableCategoryCollection, variableCategory);
        expect(expectedResult).toHaveLength(2);
      });

      it("should add a VariableCategory to an array that doesn't contain it", () => {
        const variableCategory: IVariableCategory = sampleWithRequiredData;
        const variableCategoryCollection: IVariableCategory[] = [sampleWithPartialData];
        expectedResult = service.addVariableCategoryToCollectionIfMissing(variableCategoryCollection, variableCategory);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(variableCategory);
      });

      it('should add only unique VariableCategory to an array', () => {
        const variableCategoryArray: IVariableCategory[] = [sampleWithRequiredData, sampleWithPartialData, sampleWithFullData];
        const variableCategoryCollection: IVariableCategory[] = [sampleWithRequiredData];
        expectedResult = service.addVariableCategoryToCollectionIfMissing(variableCategoryCollection, ...variableCategoryArray);
        expect(expectedResult).toHaveLength(3);
      });

      it('should accept varargs', () => {
        const variableCategory: IVariableCategory = sampleWithRequiredData;
        const variableCategory2: IVariableCategory = sampleWithPartialData;
        expectedResult = service.addVariableCategoryToCollectionIfMissing([], variableCategory, variableCategory2);
        expect(expectedResult).toHaveLength(2);
        expect(expectedResult).toContain(variableCategory);
        expect(expectedResult).toContain(variableCategory2);
      });

      it('should accept null and undefined values', () => {
        const variableCategory: IVariableCategory = sampleWithRequiredData;
        expectedResult = service.addVariableCategoryToCollectionIfMissing([], null, variableCategory, undefined);
        expect(expectedResult).toHaveLength(1);
        expect(expectedResult).toContain(variableCategory);
      });

      it('should return initial array if no VariableCategory is added', () => {
        const variableCategoryCollection: IVariableCategory[] = [sampleWithRequiredData];
        expectedResult = service.addVariableCategoryToCollectionIfMissing(variableCategoryCollection, undefined, null);
        expect(expectedResult).toEqual(variableCategoryCollection);
      });
    });

    describe('compareVariableCategory', () => {
      it('Should return true if both entities are null', () => {
        const entity1 = null;
        const entity2 = null;

        const compareResult = service.compareVariableCategory(entity1, entity2);

        expect(compareResult).toEqual(true);
      });

      it('Should return false if one entity is null', () => {
        const entity1 = { id: 123 };
        const entity2 = null;

        const compareResult1 = service.compareVariableCategory(entity1, entity2);
        const compareResult2 = service.compareVariableCategory(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey differs', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 456 };

        const compareResult1 = service.compareVariableCategory(entity1, entity2);
        const compareResult2 = service.compareVariableCategory(entity2, entity1);

        expect(compareResult1).toEqual(false);
        expect(compareResult2).toEqual(false);
      });

      it('Should return false if primaryKey matches', () => {
        const entity1 = { id: 123 };
        const entity2 = { id: 123 };

        const compareResult1 = service.compareVariableCategory(entity1, entity2);
        const compareResult2 = service.compareVariableCategory(entity2, entity1);

        expect(compareResult1).toEqual(true);
        expect(compareResult2).toEqual(true);
      });
    });
  });

  afterEach(() => {
    httpMock.verify();
  });
});
